/***************************************************************************
 *   Copyright (C) 2004 by Gerd Mueller                                    *
 *   gmueller@netways.de                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "configuration.h"

#include <kconfig.h>            // for KConfig
#include <kpanelapplet.h>
#include <kcombobox.h>
#include <qcheckbox.h>


const int  Configuration::m_defaultrefreshrate  =  1;

void Configuration::read() {
    //KConfig *conf=KPanelApplet::config();
    conf->setGroup("General");
    m_User=conf->readEntry("User","nagiosadmin");
    m_Password=conf->readEntry("Password","nagios");
    m_URL=conf->readEntry("URL", "http://");
    m_CGIURL=conf->readEntry("CGIURL","http://");
    
    m_disabled=conf->readBoolEntry("disabled",false);

    m_GroupType=conf->readNumEntry("GroupType",0);
    m_Group=conf->readEntry("Group","");
    m_HardOnly=conf->readBoolEntry("Hard",true);
    m_ACK=conf->readBoolEntry("ACK",true);
    m_SCHDWN=conf->readBoolEntry("SCHDWN",true);
    
    m_NagiosVersion=conf->readNumEntry("NagiosVersion",1);
    m_KTLightVersion=conf->readNumEntry("KTLightVersion",0);
    
    conf->setGroup("Refresh");
    m_refreshrate=conf->readNumEntry("refreshrate",m_defaultrefreshrate);

    conf->setGroup("Sound");
    m_OK=conf->readEntry("OK","");
    m_Warning=conf->readEntry("Warning","");
    m_Critical=conf->readEntry("Critical","");
    
    if(m_KTLightVersion==0) {
      // < 0.2.2.4
      m_refreshrate*=60;
      if(m_refreshrate > 300) m_refreshrate=300;
      m_KTLightVersion=1;
    }

};

void Configuration::write() const {
    //KConfig *conf=kappKPanelApplet::config();
    conf->setGroup("General");
    conf->writeEntry("User",m_User);
    conf->writeEntry("Password",m_Password);
    conf->writeEntry("URL",m_URL);
    conf->writeEntry("CGIURL",m_CGIURL);
    
    conf->writeEntry("disabled",m_disabled);
    
    conf->writeEntry("GroupType",m_GroupType);
    conf->writeEntry("Group",m_Group);
    conf->writeEntry("Hard",m_HardOnly);
    conf->writeEntry("ACK",m_ACK);
    conf->writeEntry("SCHDWN",m_SCHDWN);
    
    conf->writeEntry("NagiosVersion",m_NagiosVersion);
    conf->writeEntry("KTLightVersion",m_KTLightVersion);

    conf->setGroup("Refresh");
    conf->writeEntry("refreshrate",m_refreshrate);

    conf->setGroup("Sound");
    conf->writeEntry("OK",m_OK);
    conf->writeEntry("Warning",m_Warning);
    conf->writeEntry("Critical",m_Critical);
};

Configuration::Configuration(){
    //read(); // read the settings or set them to the default values
};

void Configuration::SetConfig(KConfig *conf) {
    this->conf=conf;
    read();
}

Configuration& Config() {
    static Configuration conf;
    return conf;
};
