/***************************************************************************
 *   Copyright (C) 2004 by Gerd Mueller                                    *
 *   gmueller@netways.de                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#include <qpainter.h>
#include <qtimer.h>
#include <qstring.h>
#include <qregexp.h>
#include <qfile.h>
#include <qbuffer.h>
#include <qtooltip.h>
#include <qlabel.h>

// #include <kglobal.h>
#include <klocale.h>
// #include <kconfig.h>
// #include <kconfigdialog.h>
// #include <kapplication.h>
#include <kurl.h>
#include <krun.h>
#include <kio/job.h>
#include <kaudioplayer.h> 
#include <kaboutdata.h>
#include <kaboutapplication.h>

// #include <kiconloader.h>
// #include <kglobalsettings.h>
// #include <khelpmenu.h>
// #include <kpopupmenu.h>
// #include <kprocess.h>
// #include <kdebug.h>

#include "prefdialog.h"		// class PrefDialog
#include "configuration.h"	// class uration and Config()

#include "ktlight.h"



KTLight::KTLight (const QString & configFile, Type type, int actions,
                  QWidget * parent, const char *name):
        KPanelApplet (configFile, type, actions, parent, name, WRepaintNoErase),QToolTip(this),m_prefDialog (0)
{

    m_isDownloading = false;
    ColorIndex = 0;
    
    m_N = new QLabel(" N",this);
    m_N->setScaledContents(TRUE);
    m_N->setBackgroundOrigin(QWidget::ParentOrigin); 
    mainView = m_N; 
    mainView->show();
    
    setmyBackgroundColor(0);

    if(Config().m_disabled) {
      m_ToolTip="KTLight is disabled.";
    } else {    
      m_ToolTip="please wait until first check has finished";
    }
    //QToolTip::add(this,m_ToolTip);

    //setFrameStyle (StyledPanel | Sunken);

    // connect to the "settingsChanged" signal
    connect (m_prefDialog, SIGNAL (settingsChanged ()), this, SLOT (applyPreferences ()));


    Config().SetConfig(config());
    applyPreferences ();


    //start first query
    emit timerEvent(NULL);     // apply the preferences


    KAboutData *about = new KAboutData("ktlight",I18N_NOOP("KTLight"),"v0.2.2.4",
                                       I18N_NOOP("An applet to visualize the overall status from nagios"),
                                       KAboutData::License_GPL_V2,
                                       "(c) 2005 Gerd Mueller");
    about->addAuthor("Gerd Mueller", I18N_NOOP("Original Author"), "gmueller@netways.de");
    m_instance = new KInstance(about);



    orientation();

}


void
KTLight::save ()
{
  m_prefDialog->updateConfiguration ();	// store settings in config object
  applyPreferences ();

  Config().write();
}

KTLight::~KTLight ()
{
}


void
KTLight::about ()
{
    KAboutApplication dlg(m_instance->aboutData());
    dlg.exec();

}


void
KTLight::preferences ()
{
    connect (m_prefDialog, SIGNAL (settingsChanged ()), this, SLOT (applyPreferences ()));

    // create dialog on demand
    if (m_prefDialog == 0)
        m_prefDialog = new PrefDialog (this);
    
    m_prefDialog->updateDialog();

    if (m_prefDialog->exec () == QDialog::Accepted)
    {
        m_prefDialog->updateConfiguration ();	// store settings in config object
        applyPreferences ();
    }
}


int KTLight::widthForHeight (int height) const
{

    return orientation()==Horizontal ? height / 2 : height * 2;
}

int KTLight::heightForWidth (int width) const
{
    return orientation()==Horizontal ? width * 2 : width / 2;
}

void KTLight::resizeEvent (QResizeEvent * e)
{
    e=e;
    update();
}

//TimerEvent to Start Geting NagiosOpenProblems
void KTLight::timerEvent (QTimerEvent *)
{

  if ((m_isDownloading) || (Config().m_URL=="http://") || Config().m_disabled)
        return;

    m_nagiosResult = "";
    m_downloadData.setBuffer(m_nagiosResult);
    m_downloadData.open (IO_WriteOnly);

    KIO::TransferJob * job = KIO::get (m_URL, false, false);
    QObject::connect (job, SIGNAL (data (KIO::Job *, const QByteArray &)), this,
                      SLOT (slotData (KIO::Job *, const QByteArray &)));
    QObject::connect (job, SIGNAL (result (KIO::Job *)), this,
                      SLOT (slotJobFinished (KIO::Job *)));

    m_isDownloading = true;
}


//Store data during GET
void KTLight::slotData (KIO::Job * job, const QByteArray & data)
{
    job=job;

    if (data.isEmpty ())
        return;

    m_downloadData.writeBlock(data.data(), data.size ());
}

//GEt finished -> examine the NagiosWebsite and set TrafficLight
void KTLight::slotJobFinished (KIO::Job * job)
{
    QString warning;
    QString critical;
    QString service_for="";
    int OldColorIndex=ColorIndex;
    bool found_nagios=FALSE;

    if (job->error ())
    {
        job->showErrorDialog ();
        m_isDownloading = false;
        return;
    }

    if (m_nagiosResult.length () > 0) {
        int i,j,l;
        int BlockColorIndex=1;
        i=0;j=0;l=0;
        QString line;
        bool insideBlock=false;
        QRegExp rx("(\\d+)/(\\d+)</TD>");
        ColorIndex=1;
        m_ToolTip="everything is in order";
        QString host="";
        QString service="";
        QString msg="";
        QString status="";
        QString soft,hard;
        bool hardstate=TRUE;
	bool real_hardstate=TRUE;



        while ((i=m_nagiosResult.find('\n',j))>=0) {
            line=m_nagiosResult.mid(j,i-j);
            j=i+1;
            
            if((line.find(QRegExp("</TR></TABLE></TD>"),0)>=0) || insideBlock) {
                l++;
                if(l==6) {
                    //get plugin output
                    status=line.replace(QRegExp("<[^>]*>"),"");
                }

                // onlyh hardstate is interesting, so ignor softstate
                if (rx.search(line) != -1 ) {
                    soft=rx.cap(1);
                    hard=rx.cap(2);
                    if(soft!=hard) {
		    	real_hardstate=FALSE;
			if(Config().m_HardOnly) hardstate=FALSE;
                    }
                }

                if ((line.find (QRegExp ("statusCRITICAL"), 0) >= 0 || 
		     line.find (QRegExp ("statusHOSTDOWN"), 0) >= 0) && hardstate)
                {
                    // red
                    BlockColorIndex = 3;
                }
                else if ((line.find (QRegExp ("statusWARNING"), 0) >= 0) && hardstate)
                {
                    // yellow
                    BlockColorIndex = 2;
                }
                if(line.find(QRegExp("</TR>"),0)>=0 && insideBlock==true) {
                    insideBlock=false;

                    // ignore softstates for the lightcolor
                    if(BlockColorIndex>ColorIndex && hardstate) {
                            ColorIndex=BlockColorIndex;
                    }

		    if(real_hardstate) {
                        msg="HARD";
                    } else {
                        msg="SOFT";
                    }
                    msg=msg+"("+soft+"/"+hard+")";

                    //but record them in the tooltip
                    msg="<tr><td>"+msg+"</td><td>"+host+"<br>"+service+"</td><td>"+status+"</td></tr>";
                    if(BlockColorIndex==2) {
                        warning=warning+msg;
                    } else if(BlockColorIndex==3) {
                        critical=critical+msg;
                    }

                    status="";
                    hardstate=TRUE;
		    real_hardstate=TRUE;
                    BlockColorIndex=1;
                    l=0;
                } else {
                    insideBlock=TRUE;
                }
            } else  {
	    	if(line.find (QRegExp ("statusHOSTDOWN"), 0) >= 0) {
		    ColorIndex=3;
                } else if (line.find(QRegExp("Service Status Details For"), 0) >= 0) {
                  service_for="<h3><center>"+line.replace(QRegExp("<[^>]*>"),"")+"</center></h3>";
                  found_nagios=TRUE;
                }

                //Service or Host?
                if(line.find(QRegExp("<TD ALIGN=LEFT valign=center CLASS="),0)>=0) {
                    service=line.replace(QRegExp("<[^>]*>"),"");
                }else if (line.find(QRegExp("<TD align=left valign=center CLASS="),0)>=0) {
                    host=line.replace(QRegExp("<[^>]*>"),"");
                }
            }
        }
        
        if(OldColorIndex!=ColorIndex) {
            switch(ColorIndex) {
            case 1:
                if(!Config().m_OK.isEmpty()) {
                    KAudioPlayer::play( Config().m_OK);
                }
                break;
            case 2:
                if(!Config().m_Warning.isEmpty()) {
                    KAudioPlayer::play( Config().m_Warning);
                }
                break;
            case 3:
                if(!Config().m_Critical.isEmpty()) {
                    KAudioPlayer::play( Config().m_Critical);
                }
                break;
            }

        }


    }
    else
    {
        ColorIndex = 0;
    }

    if(!found_nagios) {
      ColorIndex=0;
      m_ToolTip="Nagios Status not found! Check your configuration, please.";
    } else if((!warning.isEmpty()) || (!critical.isEmpty()))     {
      m_ToolTip=service_for+"<table>";
      if(!warning.isEmpty()) m_ToolTip=m_ToolTip+"<tr><td colspan=\"3\"><b>Warning:</b></td></tr>"+warning;
      if(!critical.isEmpty())m_ToolTip=m_ToolTip+"<tr><td colspan=\"3\"><b>Critical:</b></td></tr>"+critical;
      m_ToolTip=m_ToolTip+"</table>";
    } else {
      m_ToolTip=service_for+m_ToolTip;
    }
        
    setmyBackgroundColor (ColorIndex);

    QPixmap pixmap(width(), height());
    pixmap.fill(this, 0, 0);

    QPainter painter(&pixmap);
    painter.fillRect(0, 0,width(), height(), lightColor);
    painter.end();

    bitBlt(this, 0, 0, &pixmap);
    setmyBackgroundColor(ColorIndex);
        
    if(OldColorIndex!=ColorIndex) {
        repaint();
    }
    //KAudioPlayer::play( Config().m_OK);

    m_downloadData.close ();
    m_isDownloading = false;

}


void KTLight::paintEvent (QPaintEvent *)
{
    /*
    QPixmap pixmap(width(), height());
    pixmap.fill(this, 0, 0);

    QPainter painter(&pixmap);
    painter.fillRect(0, 0,width(), height(), lightColor);
    painter.end();

    bitBlt(this, 0, 0, &pixmap);
    setBackgroundColor(lightColor);
    */
}



void KTLight::executePreferencesDlg ()
{
    // create dialog on demand
    if (m_prefDialog == 0) {
        m_prefDialog = new PrefDialog (this);
        m_prefDialog->updateDialog();
    }
    if (m_prefDialog->exec () == QDialog::Accepted)
        m_prefDialog->updateConfiguration();
        applyPreferences ();
};

void KTLight::applyPreferences ()
{
    QString group;
    QString ack="61";
    QString servicetype;
    
    if(Config().m_ACK) {
      if(Config().m_SCHDWN) {
        ack="10";
      } else {
        ack="8";
      }
    } else {
      if(Config().m_SCHDWN) {
        ack="2";
      } else {
        ack="0";
      }
    }
    servicetype="hostprops="+ack+"&servicestatustypes=61&serviceprops="+ack;
    
    if(Config().m_GroupType==1) {
        //Hostgroup
	group="&style=detail&hostgroup="+Config().m_Group;
    } else if (Config().m_GroupType==2) {
        //Servicegroup
	group="&style=detail&servicegroup="+Config().m_Group;
    }
    m_BaseURL=Config().m_URL;
    m_BaseURL.setUser (Config().m_User);
    m_BaseURL.setPass (Config().m_Password);

    m_URL=Config().m_CGIURL+"status.cgi?host=all&type=detail&"+servicetype+"&noheader=1&embedded=1"+group;
    m_URL.setUser (Config().m_User);
    m_URL.setPass (Config().m_Password);

    m_NagiosURL=m_BaseURL;
    m_NagiosURL=Config().m_CGIURL+"status.cgi?host=all&type=detail&"+servicetype+group;
    m_NagiosURL.setUser (Config().m_User);
    m_NagiosURL.setPass (Config().m_Password);

    killTimer (myTimer);
    myTimer = startTimer (Config ().m_refreshrate * 1000);
    emit timerEvent(NULL);     // apply the preferences
};



void KTLight::mousePressEvent(QMouseEvent *event)
{
    if (event == 0) return;

    switch (event->button()) {
    case LeftButton: {
            KRun::runURL(m_BaseURL,"text/html");
            break;
        }
    case MidButton:
      if(Config().m_disabled) { 
        Config().m_disabled=false;
        m_N->setText(" N");
        emit timerEvent(NULL);     // apply the preferences
      } else {
        Config().m_disabled=true;
        m_N->setText(" n");
        setmyBackgroundColor(0);
        m_ToolTip="KTLight is disabled.";
      }
//        KRun::runURL(m_NagiosURL,"text/html");
        break;
    case RightButton:
        executePreferencesDlg();
        break;
    case NoButton:
    case ShiftButton:
    case ControlButton:
    case AltButton:
    case MetaButton:
    case KeyButtonMask:
    case Keypad:
    case MouseButtonMask:
        break;
    }
}

void KTLight::maybeTip ( const QPoint & pos ) {

    tip(QRect(pos,QSize(1,1)), m_ToolTip);
    //tip(r,m_ToolTip);
}

void KTLight::setmyBackgroundColor(int ColorIndex) {
  QString colors[] = { "#FFA500", "#33ff00", "#FFFF00", "#f83838","#2C4194"};
  lightColor.setNamedColor (colors[ColorIndex]);
  setBackgroundColor (lightColor);
  m_N->setBackgroundColor (lightColor);
}

// The Main

extern "C"
{
    KPanelApplet *init (QWidget * parent, const QString & configFile)
    {
        KGlobal::locale ()->insertCatalogue ("ktlight");

        KTLight * ktlight = new KTLight (configFile, KPanelApplet::Stretch,
                                         KPanelApplet::About | KPanelApplet::Preferences, parent, "ktlight");
        return ktlight;
    }
}
