/***************************************************************************
 *   Copyright (C) 2004 by Gerd Mueller                                    *
 *   gmueller@netways.de                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qlayout.h>        // for QVBoxLayout
#include <qlabel.h>         // for QLabel
#include <qframe.h>         // for QFrame
#include <qstring.h>
#include <qslider.h> 
#include <qlineedit.h>
#include <qspinbox.h>
#include <qlistbox.h>
#include <qcheckbox.h>

#include <kcolorbutton.h>   // for KColorButton
#include <kpushbutton.h>    // for KPushButton
#include <kmessagebox.h>
#include <klocale.h>        // for i18n()
#include <kiconloader.h>    // for KIconLoader
#include <kglobal.h>        // for KGlobal
#include <kpassdlg.h>        // for KGlobal
#include <kurlrequester.h>
#include <kcombobox.h>

#include "prefdialog.h"     // class PrefDialog
#include "prefdialog.moc"

#include "configuration.h"  // class Configuration and Config()
#include "prefgeneral.h"    // class PrefGeneral
#include "prefrefresh.h"
#include "prefsound.h"

#include "ktlight.h"


PrefDialog::PrefDialog(QWidget *parent, const char *name, WFlags f)
        : KDialogBase(IconList, i18n("Preferences"), Default|Ok|Apply|Cancel, Ok, parent, name, f){
    // adding page "General options"
    QFrame *frame = addPage(i18n("Nagios"), i18n("Nagios"),
                            KGlobal::iconLoader()->loadIcon("konqueror",KIcon::Panel,0,false) );
    QVBoxLayout *frameLayout = new QVBoxLayout( frame, 0, 0 );
    m_prefGeneral = new PrefGeneral(frame);
    frameLayout->addWidget(m_prefGeneral);

    // adding page "Refresh settings"
    frame = addPage(i18n("Refresh"), i18n("Refresh"),
                    KGlobal::iconLoader()->loadIcon("kalarm",KIcon::Panel,0,false) );
    frameLayout = new QVBoxLayout( frame, 0, 0 );
    m_prefRefresh = new PrefRefresh(frame);
    frameLayout->addWidget(m_prefRefresh);

    // adding page "Refresh settings"
    frame = addPage(i18n("Sound"), i18n("Sound"),
                    KGlobal::iconLoader()->loadIcon("kmix",KIcon::Panel,0,false) );
    frameLayout = new QVBoxLayout( frame, 0, 0 );
    m_prefSound = new PrefSound(frame);
    frameLayout->addWidget(m_prefSound);

    connect( m_prefGeneral->m_User, SIGNAL(textChanged(const QString&)), this, SLOT( enableApply()));
    connect( m_prefGeneral->m_Password, SIGNAL(textChanged(const QString&)), this, SLOT( enableApply()));
    connect( m_prefGeneral->m_URL, SIGNAL(textChanged(const QString&)), this, SLOT( enableApply()));
    connect( m_prefGeneral->m_CGIURL, SIGNAL(textChanged(const QString&)), this, SLOT( enableApply()));
    
    connect( m_prefGeneral->m_GroupType, SIGNAL(activated(int)), this, SLOT(GroupTypeChanged(int)));
    connect( m_prefGeneral->m_Group, SIGNAL(textChanged(const QString&)), this, SLOT( enableApply()));
    connect( m_prefGeneral->m_HardOnly, SIGNAL(toggled(bool)), this, SLOT( enableApply()));
    connect( m_prefGeneral->m_ACK, SIGNAL(toggled(bool)), this, SLOT( enableApply()));
    connect( m_prefGeneral->m_SCHDWN, SIGNAL(toggled(bool)), this, SLOT( enableApply()));

    connect( m_prefGeneral->m_NagiosVersion, SIGNAL(activated(int)), this, SLOT(VersionChanged(int)));
    
    connect( m_prefRefresh->m_refreshrate, SIGNAL( valueChanged(int) ), this, SLOT( enableApply()));
    connect( m_prefRefresh->m_refreshrate_slider, SIGNAL(valueChanged(int)), this, SLOT( enableApply()));

    connect( m_prefSound->m_OK, SIGNAL(textChanged(const QString&)), this, SLOT( enableApply()));
    connect( m_prefSound->m_Warning, SIGNAL(textChanged(const QString&)), this, SLOT( enableApply()));
    connect( m_prefSound->m_Critical, SIGNAL(textChanged(const QString&)), this, SLOT( enableApply()));
    

}

void PrefDialog::updateDialog() {
    m_prefGeneral->m_User->setText(Config().m_User);
    m_prefGeneral->m_Password->setText(Config().m_Password);
    m_prefGeneral->m_URL->setText(Config().m_URL);
    m_prefGeneral->m_CGIURL->setText(Config().m_CGIURL);

    m_prefGeneral->m_GroupType->setCurrentItem(Config().m_GroupType);
    m_prefGeneral->m_Group->setText(Config().m_Group);
    m_prefGeneral->m_HardOnly->setChecked(Config().m_HardOnly);
    m_prefGeneral->m_ACK->setChecked(Config().m_ACK);
    m_prefGeneral->m_SCHDWN->setChecked(Config().m_SCHDWN);
    
    m_prefGeneral->m_NagiosVersion->setCurrentItem(Config().m_NagiosVersion);
    
    m_prefRefresh->m_refreshrate_slider->setValue(Config().m_refreshrate );

    m_prefSound->m_OK->setURL(Config().m_OK);
    m_prefSound->m_Warning->setURL(Config().m_Warning);
    m_prefSound->m_Critical->setURL(Config().m_Critical);
    if(Config().m_GroupType>0) {
        m_prefGeneral->m_Group->setEnabled(TRUE);
    } else {
        m_prefGeneral->m_Group->setEnabled(FALSE);
    }


    enableButtonApply(false);   // disable apply button
};


void PrefDialog::updateConfiguration() {
    Config().m_User=m_prefGeneral->m_User->text();
    Config().m_Password=m_prefGeneral->m_Password->text();
    Config().m_URL=m_prefGeneral->m_URL->text();
    Config().m_CGIURL=m_prefGeneral->m_CGIURL->text();
    if(!Config().m_CGIURL.endsWith("/")) {
      Config().m_CGIURL+="/";
      m_prefGeneral->m_CGIURL->setText(Config().m_CGIURL);
    }
    
    Config().m_GroupType=m_prefGeneral->m_GroupType->currentItem();
    Config().m_Group=m_prefGeneral->m_Group->text();
    Config().m_HardOnly=m_prefGeneral->m_HardOnly->isOn();
    Config().m_ACK=m_prefGeneral->m_ACK->isOn();
    Config().m_SCHDWN=m_prefGeneral->m_SCHDWN->isOn();
    
    Config().m_NagiosVersion=m_prefGeneral->m_NagiosVersion->currentItem();
    
    Config().m_OK=m_prefSound->m_OK->url();
    Config().m_Warning=m_prefSound->m_Warning->url();
    Config().m_Critical=m_prefSound->m_Critical->url();


    Config().m_refreshrate = m_prefRefresh->m_refreshrate_slider->value();
    Config().write();
    enableButtonApply(false);   // disable apply button
};

void PrefDialog::slotDefault() {
    if (KMessageBox::warningContinueCancel(this, i18n("This will set the default options "
                                           "in ALL pages of the preferences dialog! Continue?"), i18n("Set default options?"),
                                           i18n("Set defaults"))==KMessageBox::Continue)
    {
      m_prefGeneral->m_GroupType->setCurrentItem(0);
	m_prefGeneral->m_Group->clear();
        m_prefGeneral->m_Group->setEnabled(FALSE);
        m_prefGeneral->m_HardOnly->setChecked(TRUE);
	m_prefGeneral->m_ACK->setChecked(TRUE);
        
        m_prefGeneral->m_NagiosVersion->setCurrentItem(1);
        
	m_prefRefresh->m_refreshrate_slider->setValue(1);
        

        enableApply();   // enable apply button
    }
};

void PrefDialog::enableApply() {
    enableButtonApply(true);   // enable apply button
};

void PrefDialog::GroupTypeChanged(int currentItem) {
    if(currentItem==0) {
        m_prefGeneral->m_Group->clear();
        m_prefGeneral->m_Group->setEnabled(FALSE);
    } else {
        m_prefGeneral->m_Group->setEnabled(TRUE);
    }
      
    enableApply();   // enable apply button
};

void PrefDialog::VersionChanged(int currentItem) {
  if(currentItem==0) {
    // Nagios V1.X
    //m_prefGeneral->m_ACK->setEnabled(FALSE);
  } else {
    // Nagios V2.0
    //m_prefGeneral->m_ACK->setEnabled(TRUE);
  }
      
  enableApply();   // enable apply button
};

void PrefDialog::slotApply() {
    updateConfiguration();      // transfer settings to configuration object
    emit settingsChanged();     // apply the preferences
    enableButtonApply(false);   // disable apply button again
};


